
/*
// $Source: /home/robert/kbattery/src/RCS/kbattery.cpp,v $
// $Revision: 1.1 $
// $Date: 1998/02/02 13:28:42 $
//
// Copyright (c) 1998 Robert Bradford
// Released under the terms of the GNU General Public License
// (The file COPYING in this package contains the terms of the license)
*/

#include "main.h"
#include "kbattery.h"
#include <kapp.h>
#include <qpixmap.h>
#include <stdio.h>
#include <stdlib.h>
#include <qlayout.h>
#include <qpushbt.h>
#include <qmsgbox.h>
#include "battery.xpm"
#include <klocale.h>
#include <unistd.h>
#include "kbattery.moc"

KBattery::KBattery(QWidget *parent, const char *name) : KLabel(parent,name)
  {
  QPixmap *pixmap = new QPixmap((const char **) battery_xpm);
  setPixmap(*pixmap);
  setAutoResize(TRUE);
  textlabel = new KLabel(this);
  textlabel->setAutoResize(TRUE);
  textlabel->setBackgroundColor(QColor(255,255,0));
  textlabel->setFont(QFont("helvetica",10,QFont::Normal));
  QObject::connect(this, SIGNAL(clicked()), this, SLOT(options()));
  QObject::connect(textlabel, SIGNAL(clicked()), this, SLOT(options()));

  warnlevel = 30;
  checkinterval = 10;
  KConfig *config = app->getConfig();
  QString s;
  config->setGroup("General");
  s = config->readEntry("warnlevel");
  if (!s.isNull()) warnlevel = atoi(s.data());
  s = config->readEntry("checkinterval");
  if (!s.isNull()) checkinterval = atoi(s.data());

  timer = new QTimer(this);
  connect(timer, SIGNAL(timeout()), SLOT(update()));
  timer->start(1000 * checkinterval);
  }

void KBattery::update()
  {
  char *p, buffer[100];
  int status, x;
  FILE *fp;
  static int warned = 0;
  QString s;
  
  fp = fopen("/proc/apm", "r");
  if (fp == NULL)
    {
    KApplication::beep();
    QMessageBox::critical(0, klocale->translate("Error"), 
      klocale->translate("Can't open /proc/apm"));
    sleep(5);
    exit(1);
    }
  fseek(fp, 0L,0);
  fgets(buffer, 100, fp);
  p = strchr(buffer, '%');
  if (p == NULL)
    {
    KApplication::beep();
    QMessageBox::critical(0, klocale->translate("Error"), 
      klocale->translate("Can't find battery status"));
    sleep(5);
    exit(1);
    }
  p[1] = 0;
  while (p && (p > buffer) && (*p != ' ')) p--;
  p++;
  status = atoi(p);
  fclose(fp);

  if (status < 0) status = 100;
  if (status == 100) 
    textlabel->setText(klocale->translate("AC"));
  else {
    sprintf(buffer, "%d%%", status);
    textlabel->setText(buffer);
    }
  
  if (status < warnlevel && !warned)
    {
    warned = 1;
    KApplication::beep();
    QMessageBox::warning(0, klocale->translate("Warning"), 
      klocale->translate("Low Battery"));
    }
  if (status > warnlevel && warned)
    warned = 0;
  x = (width() - textlabel->width()) / 2;
  textlabel->move(x + 1, 10);
  }

void KBattery::options()
  {
  optionsDialog = new QWidget;
  optionsDialog->setCaption(klocale->translate("KBattery Information"));
  QVBoxLayout *vbox = new QVBoxLayout(optionsDialog, 5);
  QLabel *copyright = new QLabel(optionsDialog);
  copyright->setText("Copyright (c) 1998 Robert Bradford (robert@digit.demon.co.uk)");
  vbox->addWidget(copyright, 5);
  QLabel *text = new QLabel(optionsDialog);
  vbox->addWidget(text,5);
  text->setText(klocale->translate("KBattery displays the battery level on a laptop computer."));
  QHBoxLayout *hbox1 = new QHBoxLayout;
  vbox->addLayout(hbox1, 5);
  hbox1->addStretch(2);
  QLabel *label1 = new QLabel(optionsDialog);
  hbox1->addWidget(label1,15);
  label1->setText(klocale->translate("Warning level"));
  label1->setAlignment(AlignRight);
  warnLevelText = new QLineEdit(optionsDialog);
  warnLevelText->setMaxLength(5);
  QString s;
  s.setNum(warnlevel);
  warnLevelText->setText(s);
  hbox1->addWidget(warnLevelText,5);
  QLabel *unit1 = new QLabel(optionsDialog);
  unit1->setText("(%)");
  unit1->setAlignment(AlignLeft);
  hbox1->addWidget(unit1,5);
  QLabel *label2 = new QLabel(optionsDialog);
  hbox1->addWidget(label2, 15);
  label2->setText(klocale->translate("Check Interval"));
  label2->setAlignment(AlignRight);
  intervalText = new QLineEdit(optionsDialog);
  intervalText->setMaxLength(5);
  QString t;
  t.setNum(checkinterval);
  intervalText->setText(t);
  hbox1->addWidget(intervalText,5);
  QLabel *unit2 = new QLabel(optionsDialog);
  unit2->setText(klocale->translate("(secs)"));
  unit2->setAlignment(AlignLeft);
  hbox1->addWidget(unit2,6);
  hbox1->addStretch(2);
  QHBoxLayout *hbox2 = new QHBoxLayout;
  vbox->addLayout(hbox2, 5);
  hbox2->addStretch(5);
  QPushButton *closebutton = new QPushButton(optionsDialog);
  closebutton->setText(klocale->translate("Close"));
  hbox2->addWidget(closebutton,5);
  hbox2->addStretch(5);
  QObject::connect(closebutton, SIGNAL(clicked()), this, SLOT(closeoptions()));
  QPushButton *exitbutton = new QPushButton(optionsDialog);
  exitbutton->setText(klocale->translate("Exit"));
  hbox2->addWidget(exitbutton,5);
  hbox2->addStretch(5);
  QObject::connect(exitbutton, SIGNAL(clicked()), app, SLOT(quit()));
  optionsDialog->resize(380,150);
  optionsDialog->show();
  }
  
void KBattery::closeoptions()
  {
  QString s;
  int n;
  KConfig *config = app->getConfig();
  
  config->setGroup("General");
  s = warnLevelText->text();
  n = atoi(s.data());
  if (n < 10) n = 10;
  if (n > 80) n = 80;
  warnlevel = n;
  config->writeEntry("warnlevel", warnlevel);
  s = intervalText->text();
  n = atoi(s.data());
  if (n < 1) n = 1;
  if (n > 999) n = 999;
  checkinterval = n;
  timer->changeInterval(1000 * checkinterval);
  config->writeEntry("checkinterval", checkinterval);
  config->sync();
  delete optionsDialog;
  optionsDialog = 0;
  }



